# The LearningOnline Network
# Pick a student from the classlist
#
# $Id: lonpickstudent.pm,v 1.7 2003/09/22 00:48:32 www Exp $
#
# Copyright Michigan State University Board of Trustees
#
# This file is part of the LearningOnline Network with CAPA (LON-CAPA).
#
# LON-CAPA is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# LON-CAPA is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with LON-CAPA; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# /home/httpd/html/adm/gpl.txt
#
# http://www.lon-capa.org/
#

package Apache::lonpickstudent;

use strict;
use Apache::Constants qw(:common);
use Apache::loncommon;
use Apache::loncoursedata;
use Apache::lonnet;
use Apache::lonlocal;

sub handler {
    my $r = shift;
    &Apache::loncommon::content_type($r,'text/html');
    $r->send_http_header;
    return OK if $r->header_only;

# ------------------------------------------------------------ Print the screen
    $r->print(<<ENDDOCUMENT);
<html>
<head>
<title>The LearningOnline Network with CAPA</title>
</head>
ENDDOCUMENT




    &Apache::loncommon::get_unprocessed_cgi
        ($ENV{'QUERY_STRING'},['filter','form','unameelement','udomelement',
			       'roles']);
# Allowed?

    unless (($ENV{'form.roles'}) ||
            (($ENV{'request.course.id'}) && 
             (&Apache::lonnet::allowed('srm',$ENV{'request.course.id'})))) {
	$r->print('<body>No context.</body>');
        return OK;
    }

# See if filter present

    my $filter=$ENV{'form.filter'};
    $filter=~s/\W//g;
    unless ($filter) { $filter='.'; }

    my $classlist=&Apache::loncoursedata::get_classlist();
       
# --------------------------------------- There is such a user, get environment

    $r->print(&Apache::loncommon::bodytag("Selecting a User"));
    $r->print(<<ENDSCRIPT);
<script>
function gochoose(uname,udom) {
    opener.document.$ENV{'form.form'}.$ENV{'form.unameelement'}.value=uname;
    var slct=opener.document.$ENV{'form.form'}.$ENV{'form.udomelement'};
    var i;
    for (i=0;i<slct.length;i++) {
        if (slct.options[i].value==udom) { slct.selectedIndex=i; }
    }
    self.close();
}
</script>
ENDSCRIPT
    
  $r->print('<form>');
  if ((&Apache::lonnet::allowed('srm',$ENV{'request.course.id'})) &&
      (!$ENV{'form.roles'})) {
# -------------------------------------------------------- Get course personnel
    $r->print('<h3>'.$ENV{'course.'.$ENV{'request.course.id'}.'.description'}.
              '</h3>');
    my %coursepersonnel=
       &Apache::lonnet::get_course_adv_roles();
    $r->print('<table border="2">');
    foreach my $role (sort keys %coursepersonnel) {
       foreach (split(/\,/,$coursepersonnel{$role})) {
	   my ($puname,$pudom)=split(/\:/,$_);
	   $r->print('<tr><td>'.
             '<input type="button" value="Select" onClick="gochoose('.
             "'".$puname."','".$pudom."')".'" /></td><td>'.$role.'</td><td>'.
		     &Apache::loncommon::aboutmewrapper(
                          &Apache::loncommon::plainname($puname,
                          $pudom),$puname,$pudom).'</td></tr>');
	}
    }
    $r->print('</table><p>&nbsp;');
    if ($filter ne '.') {
       $r->print('<br/ >'.&mt('Name starting with').' "'.$filter.'"<br />');
    }
    $r->print('</p><p><table>');
  # ------------------------------------------------------------------ Students
    foreach (sort keys %$classlist) {
        # the following undefs are for 'domain', and 'username' respectively.
	my (undef,undef,$end,$start,$id,$section,$fullname,$status)=
            @{$classlist->{$_}};
        if ($_=~/^(\w+)\:(\w+)$/) {
	    my ($uname,$udom)=($1,$2);
	    if (($uname=~/^$filter/) || 
                ($fullname=~/^$filter/i)) {
	       $r->print('<tr><td>'.
                      '<input type="button" value="'.&mt('Select').'" onClick="gochoose('.
                      "'".$uname."','".$udom."')".'" /></td>'.
                         '<td><tt>'.$uname.'</tt></td><td> <tt>'.$udom.
                         '</tt></td><td>'.
                         &Apache::loncommon::aboutmewrapper(
                         $fullname,
                         $uname,$udom).'</td><td>'.$id.'</td><td>'.$section.
                                       '</td></tr>');
	   }
        }
    }

    $r->print('</table></p>');
  } else {
    $r->print('<h3>'.&mt('Users with Roles Assigned by').' '.
	      &Apache::loncommon::plainname($ENV{'user.name'},
					    $ENV{'user.domain'}).'</h3>');
    if ($filter ne '.') {
       $r->print('<br/ >'.&mt('Name starting with').' "'.$filter.'"<br />');
    }
    $r->print('<p><table>');
    my %users=&Apache::lonnet::get_my_roles();
    foreach (sort keys %users) {
        if ($_=~/^(\w+)\:(\w+)\:(\w+)$/) {
	    my ($uname,$udom,$urole)=($1,$2,$3);
	    my $fullname=&Apache::loncommon::plainname($uname,$udom);
	    if (($uname=~/^$filter/) || 
                ($fullname=~/^$filter/i)) {
	       $r->print('<tr><td>'.
                      '<input type="button" value="Select" onClick="gochoose('.
                      "'".$uname."','".$udom."')".'" /></td>'.
                         '<td><tt>'.$uname.'</tt></td><td><tt>'.$udom.
                         '</tt></td><td>'.
                         &Apache::loncommon::aboutmewrapper(
                         $fullname,
                         $uname,$udom).'</td><td><td>'.
			 &Apache::lonnet::plaintext($urole).
                                       '</td></tr>');
	   }
        }
    }
    $r->print('</table></p>');
  }
  $r->print('</form></body></html>');
  return OK;
} 

1;
__END__
